/*
 * \file tiimage.c
 *
 * \brief  Generates an image bootable by the ROM code
 *
 */

/* Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/
 * ALL RIGHTS RESERVED
 */

#include <stdio.h>
#include <stdlib.h>
//#include <unistd.h>

#define BOOT NONE
#define LOAD_ADDR_APPL	0x80000000
#define LOAD_ADDR_BOOT	0x402F0400

#define BINAR_TYPE_BOOT	0
#define BINAR_TYPE_APPL	1

#define BINAR_TYPE_APPL_NAME	"Application"
#define BINAR_TYPE_BOOT_NAME	"Boot loader"

#define DEST_SPI	0
#define DEST_MMC	1
#define DEST_NOR	2

#define DEST_SPI_NAME	"SPI Flash"
#define DEST_MMC_NAME	"SD Card"
#define DEST_NOR_NAME	"NOR Flash"

#define MAGIC_NUM_SPI	0xEE3355BB
#define MAGIC_NUM_MMC	0xEE3355CC
#define MAGIC_NUM_NOR	0xEE3355DD



typedef struct _ti_header_
{
    unsigned int image_size;
    unsigned int load_addr;
}ti_header;


typedef struct _ti_header_appl
{
	unsigned int magic_number;
    unsigned int image_size;
    unsigned int load_addr;
	unsigned int run_addr;

}ti_header_appl;

#define SPI
#ifdef SPI
static unsigned int tiimage_swap32(unsigned int data)
{
    unsigned int result = 0;

    result  = (data & 0xFF000000) >> 24;
    result |= (data & 0x00FF0000) >> 8;
    result |= (data & 0x0000FF00) << 8;
    result |= (data & 0x000000FF) << 24;
   
    return result;
}
#endif


int main (int argc, char *argv[])
{
    FILE *in_fp, *out_fp;
    long image_size = 0;
    ti_header		hdr;
	ti_header_appl	hdr_appl;
    int i = 0;
	unsigned int binary_type = 0,destination = 0;	
	char *applName = NULL;
	char outFileName[128];

	unsigned long load_addr = 0;
	unsigned long run_addr = 0;
	unsigned long magicNumber = 0;



    if (argc < 5)
    {
        /* expect : tiimage </path/to/boot.bin> <path/to/place/modified/boot.bin> */
        printf("Usage : \n");
        printf("tiimage takes the input image and adds the TI Image Header \n");
        printf("The resulting output is placed in the output image path\n");
        printf("Syntax: tiimage <binary type> <destination> <input image path/name> <output image path/name> <load address in hex> <run address in hex>\n");
		printf("<binary type> -\n\t[0 - Bootloader]\n\t[1 - Application]\n");
		printf("<destination> -\n\t[0 - SPI]\n\t[1 - MMCSD]\n\t[2 - NOR]\n");
		printf("\n\tExample -\n tiimage 1 0 \"C:\\appl.bin\" \"C:\\appl_SPI.bin\" 80000000 80000000 \n");
        return -1;
    }

	binary_type = atoi(argv[1]);

	if(binary_type > 1)
	{
        printf("Error - Unknown Binary type!\n");
        return -1;
	}


	destination = atoi(argv[2]);
	if(destination > 2)
	{
        printf("Error - Unknown Destinaion!\n");
        return -1;
	}

	printf("Input File - %s \n",argv[3]);
    in_fp = fopen(argv[3], "rb");
    if(!in_fp) {
        printf("Error opening input image file!\n");
        return -1;
    }

	if(destination == DEST_MMC)
	{
		sprintf(outFileName,"%s_SD.bin",argv[4]);
		magicNumber = MAGIC_NUM_MMC;
	}
	else if(destination == DEST_SPI)
	{
		sprintf(outFileName,"%s_SPI.bin",argv[4]);
		magicNumber = MAGIC_NUM_SPI;
	}
	else
	{
		sprintf(outFileName,"%s_NOR.bin",argv[4]);
		magicNumber = MAGIC_NUM_NOR;
	}
	
	printf("Output File - %s \n",outFileName);

    out_fp = fopen(outFileName, "wb+");
    if(!out_fp) {
        printf("Error opening/creating out image file!\n");
        return -1;
    }

    if (argc > 5)
    {
		/*
		 * Get the load address in hex format. If this is not provided, default value will be used.
		 */
		sscanf(argv[5],"%x",&load_addr);
	}

	if (argc > 6)
    {
		/*
		 * Get the run address in hex format. If this is not provided, default value will be used..
		 */
		sscanf(argv[6],"%x",&run_addr);
	}


	printf("Generating TI_Image of [%s] to be stored in ",
		((binary_type == BINAR_TYPE_APPL)?BINAR_TYPE_APPL_NAME:BINAR_TYPE_BOOT_NAME));

	if(destination == DEST_MMC)
		printf("[%s]\n",DEST_MMC_NAME);
	else if(destination == DEST_SPI)
		printf("[%s]\n",DEST_SPI_NAME);
	else
		printf("[%s]\n",DEST_NOR_NAME);


    /* Calcualte the size of the input image and rewind to the begin of file */
    fseek(in_fp, 0, SEEK_END);
    image_size = ftell(in_fp);
    rewind(in_fp);

	if(binary_type == BINAR_TYPE_APPL)
	{
		/* Binary Type - Application */
		hdr_appl.magic_number = magicNumber;
		/* Size of  new image is actual bin image size + header */
		hdr_appl.image_size = image_size + sizeof(hdr_appl);
		hdr_appl.load_addr = ((load_addr == 0)?LOAD_ADDR_APPL:load_addr);
		hdr_appl.run_addr = ((run_addr == 0)?LOAD_ADDR_APPL:run_addr);

		/* Insert the header first */
		fwrite(&hdr_appl, sizeof(hdr_appl), 1, out_fp);
	}
	else
	{	
		/* Binary Type - Boot laoder */
		if(destination == DEST_SPI)
		{
			/* Swap required */
			/* Size of  new image is actual bin image size + header */
			hdr.image_size = tiimage_swap32(image_size + sizeof(hdr));
			hdr.load_addr = tiimage_swap32(((load_addr == 0)?LOAD_ADDR_BOOT:load_addr));
			/* Insert the header first */
			fwrite(&hdr, sizeof(hdr), 1, out_fp);
		}
		else if(destination == DEST_MMC)
		{
			hdr.image_size = image_size + sizeof(hdr);
			hdr.load_addr = ((load_addr == 0)?LOAD_ADDR_BOOT:load_addr);
			/* Insert the header first */
			fwrite(&hdr, sizeof(hdr), 1, out_fp);
		}

	}

    /* Insert the actual image */
    for (i = 0; i <= (image_size / 4); i++) 
	{
        unsigned int temp;
        fread(&temp, sizeof(temp), 1, in_fp);

		if((binary_type == BINAR_TYPE_BOOT) &&
		   (destination == DEST_SPI))
		{
			temp = tiimage_swap32(temp);
		}

        fwrite(&temp, sizeof(temp), 1, out_fp);
    }

    printf("\n");

    return 0;
}
